/**
 *    _____ __
 *   / ___// /_  _____
 *   \__ \/ / / / / _ \
 *  ___/ / / /_/ /  __/
 * /____/_/\__, /\___/
 *       /____/
 *       Copyright 2017 Slye Development Team. All Rights Reserved.
 */
function timestamp(){
    return Math.floor((new Date()).getTime() / 1000)
}

let $app = angular.module('slye', [
	'ngRoute',
    'slye.auth',
    'slye.dashboard'
]);

$app.config($routeProvider, $locationProvider){
    $locationProvider.html5Mode(true)
	$routeProvider.otherwise({redirectTo: '/dashboard'});
}

$app.run($rootScope, $timeout, $location, $interval, slye){
	#$rootScope<'$routeChangeStart'>(event, next, prev).$on{
        if (next && next.$$route) {
            $timeout{
				$rootScope.page = {};
	            $rootScope.page.controller  = next.$$route.controller;
	            $rootScope.page.params      = next.pathParams;
                // Don't allow user to see auth page after login
                if($rootScope.profile && ['LoginCtrl', 'RegisterCtrl', 'VerifyCtrl', 'ForgetReqCtrl', 'ForgetChCtrl'].indexOf(next.$$route.controller) > -1){
                    $location.path('/');
                }
			}
        }
    }
}

$app.directive<"compareTo">(){
    return {
        require: "ngModel",
        scope: {
            otherModelValue: "=compareTo"
        },
        link: function(scope, element, attributes, ngModel) {

            ngModel.$validators.compareTo = function(modelValue) {
                return modelValue == scope.otherModelValue;
            };

            scope.$watch("otherModelValue", function() {
                ngModel.$validate();
            });
        }
    };
}

$app.factory<'slye'>($http, $rootScope, $timeout, $location){
    const app = {
        time    : 5 * 60,
        host    : 'http://127.0.0.1:2030/api',
        id      : 'slye.web.demo',
        token   : [
            'N5XdHyWtVqlwAplJZ51Eum5UvS7E39jSY1A7MVcCI0tVOfRml9LDtToApM9lffiB',
            'qHG0sYw9FBJ46nau4xy1KV7V70S24w7YSIerCNb5z9XavghXo9t9fmpNrCap39Ci',
            'XwShfd4Fg13annuclOux1kp3HF2CBVZxEtJChgOlmdvvzz9XfZVXNtone0eSWhVM',
            '6luEiANd2tr9rYG1gkdI16SpNjP3kMDoYJN2gji7CqtKE6TIr9NjPWZTGXZiD6G6',
            'NuoiFsv8KxQOqZWXdDf6zWMQy3mOcS6iY0QZdAveUF9oBOzdcE5huEGmNoLiczhB',
            'RMot8TduA2f9wCNWEDQiDbnwgyNxAPo2fwCx7cVUEksz2hG8aX4Rs85imGLRvlWl',
            'Un3i35VCytNSInj1Drpe3jIhFOX1bo0MS2f2Vy9zFLlqxlCEXCyILxI8QdS34PDK',
            '4aq9h9ehTp9jBvQ4zeerWORu4xicAA4yPyXDlRWnuYh0zoBCOZpqyfNfigiAGYIN'
        ].join('')
    };
    let token;
    let storage = localStorage;

    /**
     * Send request to the Slye server.
     * @param  string   endpoint    API EndPoint name
     * @param  object   data        Data to be send,
     *    This function adds user token automtically so there is no need to insert
     *    it yourslef
     * @param   boolean code200     In the case this is true, this function will
     *     call reject when response code is not 200
     * @return promise  Callback result
     */
    promise slye(endpoint, _data, code200)<resolve, reject>{
        code200 = code200 ? true : false;
        let random  = Math.round(Math.random() * 2500);
        let key = slye.key(random)
        if(!_data)
            _data   = {}
        if(token)
            _data['token']   = token;
        _data['endpoint']   = endpoint;
        var data = $.param({
            'data'      : slye._encode(JSON.stringify(_data), key),
            'app.id'    : app.id,
            'random'    : random
        });

        var config = {
            headers : {
                'Content-Type': 'application/x-www-form-urlencoded'
            }
        }

        $http.post(app.host, data, config).then(function ({data, status, headers, config}) {
            let random2 = headers('SlyeProto-Key');
            if(random2){
                random2 = slye._decode(random2, key)
                let key2    = sha256(key + random2);
                let _data   = slye._decode(data, key2)
                let t;
                try{
                    t   = JSON.parse(_data)
                }catch(e){
                    t   = _data;
                }
                data    = t;
            }
            if(!data)
                data = {}
            if(typeof data == 'string')
                data = {msg: data}
            if(code200 && data['code'] && data['code'] !== 200){
                data['server_err']  = false;
                return reject(data);
            }
            resolve(data);
        }, function ({data, status, headers, config}) {
            let random2 = headers('SlyeProto-Key');
            if(random2){
                random2 = slye._decode(random2, key)
                let key2    = sha256(key + random2);
                let _data   = slye._decode(data, key2)
                let t;
                try{
                    t   = JSON.parse(_data)
                }catch(e){
                    t   = _data;
                }
                data    = t;
            }
            if(!data)
                data = {}
            if(typeof data == 'string')
                data = {msg: data}
            data['server_err']  = true;
            reject(data);
        });
    }

    /**
     * XOR string with given key
     * @param  string   string  String to be encrypted
     * @param  string   key     String to use as encryption key
     * @return string   encoded string
     */
    slye._xor   = function(string, key){
        let re  = '',
            le  = key.length;
        for(let i = 0;i < string.length;i++){
            re += String.fromCharCode(string.charCodeAt(i) ^ key.charCodeAt(i % le));
        }
        return re;
    }

    /**
     * Encryption function
     *  String -> XOR -> base64 encode -> encrypted data
     * @param   string  string  string String to be encoded
     * @param   string  key     XOR key
     * @return string   encoded data
     */
    slye._encode    = function(string, key){
        string  = (' '.repeat(Math.round(Math.random() * 32)))
                + string
                + (' '.repeat(Math.round(Math.random() * 32)));
        string  = slye._xor(string, key);
        string  = btoa(string);
        return string;
    }

    /**
     * Decryption function
     * String -> base64 decode -> XoR -> decrypted data
     * @param  string   string encoded string
     * @param  string   key    XOR key
     * @return string   decoded data
     */
    slye._decode    = function(string, key){
        string  = atob(string)
        string  = slye._xor(string, key);
        string  = string.trim()
        return string;
    }

    slye.key    = function(random){
        return sha256(JSON.stringify([random + '',app.token, Math.round(timestamp() / (app.time)) * app.time]));
    }

    slye.token  = function(_token){
        token = _token;
        storage.setItem('token', token)
        wait for slye('auth.me') as profile{
            $timeout{
                $rootScope.profile  = profile;
                if($rootScope.profile && ['LoginCtrl', 'RegisterCtrl', 'VerifyCtrl', 'ForgetReqCtrl', 'ForgetChCtrl'].indexOf($rootScope.page.controller) > -1){
                    $location.path('/');
                }
            }
        } error(e){
            $rootScope.profile  = false;
        }
    }

    if(storage.getItem('token')){
        slye.token(storage.getItem('token'))
    }
    return slye;
}

let $doc    = $(document);
$doc.ready(function(){
    /** $('.navbar-link').click(function(){
         $(this).toggleClass('is-active');
    })*/

    $('.navbar-burger').click(function(){
        $('#' + this.dataset.target).toggleClass('is-active')
    })
})
